#include <tadah/core/element.h>
#include <tadah/mlip/structure.h>
#include <tadah/mlip/structure_db.h>
#include <tadah/mlip/dataset_writers/castep_cell_writer.h>

namespace tadah {
namespace mlip {

CastepCellWriter::CastepCellWriter() : DatasetWriter() {}

void CastepCellWriter::write_data(const StructureDB &stdb, const std::string& filename, size_t structIndex) {

  if (structIndex >= stdb.size()) {
    throw std::out_of_range("Index structIndex is out of range.");
  }
    
  std::ofstream file(filename);
  if (!file.is_open()) {
    throw std::runtime_error("Could not open the file: " + filename);
  }

  const Structure &st = stdb(structIndex);

  // write label
  file << "# " << st.label << std::endl;

  // write cell
  file << "%BLOCK LATTICE_CART" << std::endl;
  file << "ANG" << std::endl;
  for (int i=0; i<3; ++i) {
    file << std::right << std::fixed << std::setw(w)
      << std::setprecision(p) << st.cell(i,0);
    file << std::right << std::fixed << std::setw(w)
      << std::setprecision(p) << st.cell(i,1);
    file << std::right << std::fixed << std::setw(w)
      << std::setprecision(p) << st.cell(i,2);
    file << std::endl;
  }
  file << "%ENDBLOCK LATTICE_CART" << std::endl;

  file << std::endl;

  file << "%BLOCK POSITIONS_ABS" << std::endl;
  for (const auto &atom : st) {
    std::string symbol = tadah::core::Element::toSymbol(atom.atomicNumber());
    file << std::right << std::fixed << std::setw(w)
      << symbol;
    file << std::right << std::fixed << std::setw(w)
      << std::setprecision(p) << atom.x();
    file << std::right << std::fixed << std::setw(w)
      << std::setprecision(p) << atom.y();
    file << std::right << std::fixed << std::setw(w)
      << std::setprecision(p) << atom.z();
    file << std::endl;
  }
  file << "%ENDBLOCK POSITIONS_ABS	" << std::endl;
  file.close();
}
}
}
