#include <tadah/mlip/design_matrix/functions/basis_functions/dm_bf_linear.h>

namespace tadah {
namespace mlip {
DM_BF_Linear::DM_BF_Linear() {}
DM_BF_Linear::DM_BF_Linear(const tadah::core::Config &c)
    : tadah::models::Function_Base(c), DM_BF_Base(c), tadah::models::BF_Linear(c) {}
size_t DM_BF_Linear::get_phi_cols(const tadah::core::Config &config) {
  size_t cols = config.get<size_t>("DSIZE");
  return cols;
}
void DM_BF_Linear::calc_phi_energy_row(tadah::core::phi_type &Phi, size_t &row,
                                       const double fac, const Structure &,
                                       const StructureNeighborView &st_nb,
                                       const StDescriptors &st_d) {
  for (size_t a = 0; a < st_d.naed(); ++a) {
    const tadah::core::aed_type &aed = st_d.get_aed(a);
    for (size_t j = 0; j < aed.size(); ++j) {
      Phi(row, j) += aed[j] * fac;
    }
  }
  row++;
}
void DM_BF_Linear::calc_phi_force_rows(tadah::core::phi_type &Phi, size_t &row,
                                       const double fac, const Structure &st,
                                       const StructureNeighborView &st_nb,
                                       const StDescriptors &st_d) {

  for (size_t a=0; a<st.natoms(); ++a) {
    const std::size_t* NbPtr    = st_nb.getNeighborsPtr(a);
    size_t NNbrs = st_nb.numNeighbors(a);
    const tadah::core::aed_type& aedi = st_d.get_aed(a);
    for (size_t jj=0; jj<NNbrs; ++jj) {
      size_t gj = NbPtr[jj];
      size_t j= st.globalToLocal(gj);  // j is local to st
      size_t aa = st_nb.getAAIndex(a, j, jj);
      const tadah::core::fd_type &fij = st_d.fd[a][jj];
      const tadah::core::fd_type &fji = st_d.fd[j][aa];
      for (size_t k = 0; k < 3; ++k) {
        for (size_t d = 0; d < fij.rows(); ++d) {
          Phi(row + k, d) -= fac * (fij(d, k) - fji(d, k));
        }
      }
    }
    row += 3;
  }
}
void DM_BF_Linear::calc_phi_stress_rows(tadah::core::phi_type &Phi, size_t &row,
                                        const double fac[6],
                                        const Structure &st,
                                        const StructureNeighborView &st_nb,
                                        const StDescriptors &st_d) {
  double V_inv = 1 / st.get_volume();
  tadah::core::Vec3d rj;
  for (size_t a = 0; a < st.natoms(); ++a) {

    const std::size_t *NbPtr = st_nb.getNeighborsPtr(a);
    size_t NNbrs = st_nb.numNeighbors(a);
    tadah::core::Vec3dSoAConstView ra = st.positionView(a);
    const tadah::core::aed_type &aedi = st_d.get_aed(a);
    for (size_t jj = 0; jj < NNbrs; ++jj) {
      size_t gj = NbPtr[jj];
      size_t j = st.globalToLocal(gj); // j is local to st
      size_t aa = st_nb.getAAIndex(a, j, jj);

      auto sx = *st_nb.getShiftXPtr(j);
      auto sy = *st_nb.getShiftYPtr(j);
      auto sz = *st_nb.getShiftZPtr(j);

      rj(0) = st.x(j) + sx*st.cell(0,0) + sy*st.cell(0,1) + sz*st.cell(0,2);
      rj(1) = st.y(j) + sx*st.cell(1,0) + sy*st.cell(1,1) + sz*st.cell(1,2);
      rj(2) = st.z(j) + sx*st.cell(2,0) + sy*st.cell(2,1) + sz*st.cell(2,2);

      const tadah::core::fd_type &fdji = st_d.fd[j][aa];
      const tadah::core::fd_type &fdij = st_d.fd[a][jj];
      const tadah::core::aed_type &aedj = st_d.get_aed(j);

      size_t mn = 0;
      for (size_t x = 0; x < 3; ++x) {
        for (size_t y = x; y < 3; ++y) {
          for (size_t d = 0; d < fdij.rows(); ++d) {
            Phi(row + mn, d) += V_inv * (fdij(d, y) - fdji(d, y)) * 0.5 *
              fac[mn] * (ra(x) - rj(x));
          }
          mn++;
        }
      }
    }
  }
  row += 6;
}
} // namespace mlip
} // namespace tadah
