#include "tadah/mlip/design_matrix/functions/kernels/dm_kern_base.h"
#include <tadah/mlip/design_matrix/functions/kernels/dm_kern_linear.h>

//tadah::core::Registry<DM_Function_Base>::Register<DM_Kern_Linear> DM_Kern_Linear_1( "tadah::models::Kern_Linear" );
//tadah::core::Registry<DM_Function_Base,tadah::core::Config&>::Register<DM_Kern_Linear> DM_Kern_Linear_2( "tadah::models::Kern_Linear" );

namespace tadah {
namespace mlip {
DM_Kern_Linear::DM_Kern_Linear() {}
DM_Kern_Linear::DM_Kern_Linear (const tadah::core::Config &c): 
  tadah::models::Function_Base(c),
  DM_Kern_Base(c),
  tadah::models::Kern_Linear(c)
{}
size_t DM_Kern_Linear::get_phi_cols(const tadah::core::Config &config)
{
  size_t cols = config.get<size_t>("DSIZE");
  return cols;
}
void DM_Kern_Linear::calc_phi_energy_row(tadah::core::phi_type &Phi, size_t &row,
                                         const double fac, const Structure &, const StructureNeighborView &st_nb, const StDescriptors &st_d)
{
  for (size_t a=0; a<st_d.naed();++a) {
    const tadah::core::aed_type &aed = st_d.get_aed(a);  // TODO
    for (size_t j=0; j<aed.size(); ++j) {
      Phi(row,j)+=aed[j]*fac;
    }
  }
  row++;
}
void DM_Kern_Linear::calc_phi_force_rows(tadah::core::phi_type &Phi, size_t &row,
                                         const double fac, const Structure &st, const StructureNeighborView &st_nb, const StDescriptors &st_d)
{
  for (size_t a=0; a<st.natoms(); ++a) {
    const std::size_t* NbPtr = st_nb.getNeighborsPtr(a);
    size_t NNbrs = st_nb.numNeighbors(a);
    for (size_t jj=0; jj<NNbrs; ++jj) {
      const size_t gj = NbPtr[jj];
      const size_t j = st.globalToLocal(gj);  // j is local to st
      const size_t aa = st_nb.getAAIndex(a, j, jj);
      for (size_t k=0; k<3; ++k) {
        tadah::core::aed_type temp = (st_d.fd[a][jj](k)-
          st_d.fd[j][aa](k))*fac;
        for (size_t d=0; d<temp.size(); ++d) {
          Phi(row+k,d) -= temp[d];
        }
      }
    }
    row+=3;
  }

}
void DM_Kern_Linear::calc_phi_stress_rows(tadah::core::phi_type &Phi, size_t &row,
                                          const double fac[6], const Structure &st, const StructureNeighborView &st_nb, const StDescriptors &st_d)
{
  double V_inv = 1/st.get_volume();
  tadah::core::Vec3d rj;
  for (size_t a=0; a<st.natoms(); ++a) {
    const std::size_t* NbPtr = st_nb.getNeighborsPtr(a);
    size_t NNbrs = st_nb.numNeighbors(a);
    tadah::core::Vec3dSoAConstView ra = st.positionView(a);
    for (size_t jj=0; jj<NNbrs; ++jj) {
      size_t gj = NbPtr[jj];
      size_t j= st.globalToLocal(gj);  // j is local to st
      size_t aa = st_nb.getAAIndex(a, j, jj);
      const tadah::core::fd_type &fdij = st_d.fd[a][jj];
      const tadah::core::fd_type &fdji = st_d.fd[j][aa];

      auto sx = *st_nb.getShiftXPtr(j);
      auto sy = *st_nb.getShiftYPtr(j);
      auto sz = *st_nb.getShiftZPtr(j);

      rj(0) = st.x(j) + sx*st.cell(0,0) + sy*st.cell(0,1) + sz*st.cell(0,2);
      rj(1) = st.y(j) + sx*st.cell(1,0) + sy*st.cell(1,1) + sz*st.cell(1,2);
      rj(2) = st.z(j) + sx*st.cell(2,0) + sy*st.cell(2,1) + sz*st.cell(2,2);

      size_t mn=0;
      for (size_t x=0; x<3; ++x) {
        for (size_t y=x; y<3; ++y) {
          tadah::core::aed_type temp = V_inv*(fdij(y)-fdji(y))*0.5*fac[mn]*(ra(x)-rj(x));
          for (size_t d=0; d<temp.size(); ++d) {
            Phi(row+mn,d) += temp[d];
          }
          mn++;
        }
      }
    }
  }
  row += 6;
}

}
}
